@@ -38,6 +38,7 @@ extern unsigned int nb_applets;
 extern struct pool_head *pool_head_appctx;
 
 struct task *task_run_applet(struct task *t, void *context, unsigned int state);
+struct task *task_process_applet(struct task *t, void *context, unsigned int state);
 int appctx_buf_available(void *arg);
 void *applet_reserve_svcctx(struct appctx *appctx, size_t size);
 void applet_reset_svcctx(struct appctx *appctx);
@@ -48,6 +49,19 @@ int appctx_finalize_startup(struct appctx *appctx, struct proxy *px, struct buff
 void appctx_free_on_early_error(struct appctx *appctx);
 void appctx_free(struct appctx *appctx);
 
+size_t appctx_htx_rcv_buf(struct appctx *appctx, struct buffer *buf, size_t count, unsigned int flags);
+size_t appctx_raw_rcv_buf(struct appctx *appctx, struct buffer *buf, size_t count, unsigned int flags);
+size_t appctx_rcv_buf(struct stconn *sc, struct buffer *buf, size_t count, unsigned int flags);
+
+size_t appctx_htx_snd_buf(struct appctx *appctx, struct buffer *buf, size_t count, unsigned int flags);
+size_t appctx_raw_snd_buf(struct appctx *appctx, struct buffer *buf, size_t count, unsigned int flags);
+size_t appctx_snd_buf(struct stconn *sc, struct buffer *buf, size_t count, unsigned int flags);
+
+int appctx_fastfwd(struct stconn *sc, unsigned int count, unsigned int flags);
+ssize_t applet_append_line(void *ctx, struct ist v1, struct ist v2, size_t ofs, size_t len);
+static forceinline void applet_fl_set(struct appctx *appctx, uint on);
+static forceinline void applet_fl_clr(struct appctx *appctx, uint off);
+
 static inline struct appctx *appctx_new_here(struct applet *applet, struct sedesc *sedesc)
 {
 	return appctx_new_on(applet, sedesc, tid);
@@ -58,6 +72,41 @@ static inline struct appctx *appctx_new_anywhere(struct applet *applet, struct s
 	return appctx_new_on(applet, sedesc, -1);
 }
 
+
+/*
+ * Release a buffer, if any, and try to wake up entities waiting in the buffer
+ * wait queue.
+ */
+static inline void appctx_release_buf(struct appctx *appctx, struct buffer *bptr)
+{
+	if (bptr->size) {
+		b_free(bptr);
+		offer_buffers(appctx->buffer_wait.target, 1);
+	}
+}
+
+/*
+ * Allocate a buffer. If if fails, it adds the appctx in buffer wait queue and
+ * sets the relevant blocking flag depending on the side (assuming that bptr is
+ * either &appctx->inbuf or &appctx->outbuf). Upon success it will also clear
+ * the equivalent MAYALLOC flags.
+ */
+static inline struct buffer *appctx_get_buf(struct appctx *appctx, struct buffer *bptr)
+{
+	struct buffer *buf = NULL;
+	int is_inbuf = (bptr == &appctx->inbuf);
+
+	if (likely(!LIST_INLIST(&appctx->buffer_wait.list))) {
+		if (unlikely((buf = b_alloc(bptr, is_inbuf ? DB_MUX_TX : DB_SE_RX)) == NULL)) {
+			b_queue(is_inbuf ? DB_MUX_TX : DB_SE_RX, &appctx->buffer_wait, appctx, appctx_buf_available);
+			applet_fl_set(appctx, is_inbuf ? APPCTX_FL_INBLK_ALLOC : APPCTX_FL_OUTBLK_ALLOC);
+		} else {
+			applet_fl_clr(appctx, is_inbuf ? APPCTX_FL_IN_MAYALLOC : APPCTX_FL_OUT_MAYALLOC);
+		}
+	}
+	return buf;
+}
+
 /* Helper function to call .init applet callback function, if it exists. Returns 0
  * on success and -1 on error.
  */
@@ -78,9 +127,11 @@ static inline int appctx_init(struct appctx *appctx)
 /* Releases an appctx previously allocated by appctx_new(). */
 static inline void __appctx_free(struct appctx *appctx)
 {
+	appctx_release_buf(appctx, &appctx->inbuf);
+	appctx_release_buf(appctx, &appctx->outbuf);
+
 	task_destroy(appctx->t);
-	if (LIST_INLIST(&appctx->buffer_wait.list))
-		LIST_DEL_INIT(&appctx->buffer_wait.list);
+	b_dequeue(&appctx->buffer_wait);
 	if (appctx->sess)
 		session_free(appctx->sess);
 	BUG_ON(appctx->sedesc && !se_fl_test(appctx->sedesc, SE_FL_ORPHAN));
@@ -109,6 +160,67 @@ static inline struct stream *appctx_strm(const struct appctx *appctx)
 	return __sc_strm(appctx->sedesc->sc);
 }
 
+/* returns 1 if the appctx is attached on the backend side or 0 if it is
+ * attached on the frontend side. Note that only frontend appctx may have no SC.
+ */
+static inline int appctx_is_back(const struct appctx *appctx)
+{
+	struct stconn *sc = appctx_sc(appctx);
+
+	return !!(sc && (sc->flags & SC_FL_ISBACK));
+}
+
+static forceinline void applet_fl_zero(struct appctx *appctx)
+{
+	appctx->flags = 0;
+}
+
+static forceinline void applet_fl_setall(struct appctx *appctx, uint all)
+{
+	appctx->flags = all;
+}
+
+static forceinline void applet_fl_set(struct appctx *appctx, uint on)
+{
+	if (((on & (APPCTX_FL_EOS|APPCTX_FL_EOI)) && appctx->flags & APPCTX_FL_ERR_PENDING) ||
+	    ((on & APPCTX_FL_ERR_PENDING) && appctx->flags & (APPCTX_FL_EOI|APPCTX_FL_EOS)))
+		on |= APPCTX_FL_ERROR;
+	appctx->flags |= on;
+}
+
+static forceinline void applet_fl_clr(struct appctx *appctx, uint off)
+{
+	appctx->flags &= ~off;
+}
+
+static forceinline uint applet_fl_test(const struct appctx *appctx, uint test)
+{
+	return !!(appctx->flags & test);
+}
+
+static forceinline uint applet_fl_get(const struct appctx *appctx)
+{
+	return appctx->flags;
+}
+
+static inline void applet_set_eoi(struct appctx *appctx)
+{
+	applet_fl_set(appctx, APPCTX_FL_EOI);
+}
+
+static inline void applet_set_eos(struct appctx *appctx)
+{
+	applet_fl_set(appctx, APPCTX_FL_EOS);
+}
+
+static inline void applet_set_error(struct appctx *appctx)
+{
+	if (applet_fl_test(appctx, (APPCTX_FL_EOS|APPCTX_FL_EOI)))
+		applet_fl_set(appctx, APPCTX_FL_ERROR);
+	else
+		applet_fl_set(appctx, APPCTX_FL_ERR_PENDING);
+}
+
 /* The applet announces it has more data to deliver to the stream's input
  * buffer.
  */
@@ -173,20 +285,32 @@ static inline void applet_expect_data(struct appctx *appctx)
  */
 static inline int applet_putchk(struct appctx *appctx, struct buffer *chunk)
 {
-	struct sedesc *se = appctx->sedesc;
 	int ret;
 
-	ret = ci_putchk(sc_ic(se->sc), chunk);
-	if (ret < 0) {
-		/* XXX: Handle all errors as a lack of space because callers
-		 * don't handles other cases for now. So applets must be
-		 * careful to handles shutdown (-2) and invalid calls (-3) by
-		 * themselves.
-		 */
-		sc_need_room(se->sc, chunk->data);
-		ret = -1;
+	if (appctx->flags & APPCTX_FL_INOUT_BUFS) {
+		if (b_data(chunk) > b_room(&appctx->outbuf)) {
+			applet_fl_set(appctx, APPCTX_FL_OUTBLK_FULL);
+			ret = -1;
+		}
+		else {
+			ret = b_putblk(&appctx->outbuf, b_head(chunk), b_data(chunk));
+			chunk->data -= ret;
+		}
 	}
+	else {
+		struct sedesc *se = appctx->sedesc;
 
+		ret = ci_putchk(sc_ic(se->sc), chunk);
+		if (ret < 0) {
+			/* XXX: Handle all errors as a lack of space because callers
+			 * don't handles other cases for now. So applets must be
+			 * careful to handles shutdown (-2) and invalid calls (-3) by
+			 * themselves.
+			 */
+			sc_need_room(se->sc, chunk->data);
+			ret = -1;
+		}
+	}
 	return ret;
 }
 
@@ -196,18 +320,29 @@ static inline int applet_putchk(struct appctx *appctx, struct buffer *chunk)
  */
 static inline int applet_putblk(struct appctx *appctx, const char *blk, int len)
 {
-	struct sedesc *se = appctx->sedesc;
 	int ret;
 
-	ret = ci_putblk(sc_ic(se->sc), blk, len);
-	if (ret < 0) {
-		/* XXX: Handle all errors as a lack of space because callers
-		 * don't handles other cases for now. So applets must be
-		 * careful to handles shutdown (-2) and invalid calls (-3) by
-		 * themselves.
-		 */
-		sc_need_room(se->sc, len);
-		ret = -1;
+	if (appctx->flags & APPCTX_FL_INOUT_BUFS) {
+		if (len > b_room(&appctx->outbuf)) {
+			applet_fl_set(appctx, APPCTX_FL_OUTBLK_FULL);
+			ret = -1;
+		}
+		else
+			ret = b_putblk(&appctx->outbuf, blk, len);
+	}
+	else {
+		struct sedesc *se = appctx->sedesc;
+
+		ret = ci_putblk(sc_ic(se->sc), blk, len);
+		if (ret < 0) {
+			/* XXX: Handle all errors as a lack of space because callers
+			 * don't handles other cases for now. So applets must be
+			 * careful to handles shutdown (-2) and invalid calls (-3) by
+			 * themselves.
+			 */
+			sc_need_room(se->sc, len);
+			ret = -1;
+		}
 	}
 
 	return ret;
@@ -220,20 +355,32 @@ static inline int applet_putblk(struct appctx *appctx, const char *blk, int len)
  */
 static inline int applet_putstr(struct appctx *appctx, const char *str)
 {
-	struct sedesc *se = appctx->sedesc;
 	int ret;
 
-	ret = ci_putstr(sc_ic(se->sc), str);
-	if (ret < 0) {
-		/* XXX: Handle all errors as a lack of space because callers
-		 * don't handles other cases for now. So applets must be
-		 * careful to handles shutdown (-2) and invalid calls (-3) by
-		 * themselves.
-		 */
-		sc_need_room(se->sc, strlen(str));
-		ret = -1;
-	}
+	if (appctx->flags & APPCTX_FL_INOUT_BUFS) {
+		int len = strlen(str);
 
+		if (len > b_room(&appctx->outbuf)) {
+			applet_fl_set(appctx, APPCTX_FL_OUTBLK_FULL);
+			ret = -1;
+		}
+		else
+			ret = b_putblk(&appctx->outbuf, str, len);
+	}
+	else {
+		struct sedesc *se = appctx->sedesc;
+
+		ret = ci_putstr(sc_ic(se->sc), str);
+		if (ret < 0) {
+			/* XXX: Handle all errors as a lack of space because callers
+			 * don't handles other cases for now. So applets must be
+			 * careful to handles shutdown (-2) and invalid calls (-3) by
+			 * themselves.
+			 */
+			sc_need_room(se->sc, strlen(str));
+			ret = -1;
+		}
+	}
 	return ret;
 }
 
@@ -243,20 +390,32 @@ static inline int applet_putstr(struct appctx *appctx, const char *str)
  */
 static inline int applet_putchr(struct appctx *appctx, char chr)
 {
-	struct sedesc *se = appctx->sedesc;
 	int ret;
 
-	ret = ci_putchr(sc_ic(se->sc), chr);
-	if (ret < 0) {
-		/* XXX: Handle all errors as a lack of space because callers
-		 * don't handles other cases for now. So applets must be
-		 * careful to handles shutdown (-2) and invalid calls (-3) by
-		 * themselves.
-		 */
-		sc_need_room(se->sc, 1);
-		ret = -1;
+	if (appctx->flags & APPCTX_FL_INOUT_BUFS) {
+		if (b_full(&appctx->outbuf)) {
+			applet_fl_set(appctx, APPCTX_FL_OUTBLK_FULL);
+			ret = -1;
+		}
+		else {
+			 b_putchr(&appctx->outbuf, chr);
+			 ret = 1;
+		}
 	}
+	else {
+		struct sedesc *se = appctx->sedesc;
 
+		ret = ci_putchr(sc_ic(se->sc), chr);
+		if (ret < 0) {
+			/* XXX: Handle all errors as a lack of space because callers
+			 * don't handles other cases for now. So applets must be
+			 * careful to handles shutdown (-2) and invalid calls (-3) by
+			 * themselves.
+			 */
+			sc_need_room(se->sc, 1);
+			ret = -1;
+		}
+	}
 	return ret;
 }
 
