@@ -99,7 +99,7 @@ const struct http_hdr hpack_sht[HPACK_SHT_SIZE] = {
 	[61] = { .n = IST("www-authenticate"),             .v = IST("")              },
 };
 
-struct pool_head *pool_head_hpack_tbl = NULL;
+struct pool_head *pool_head_hpack_tbl __read_mostly = NULL;
 
 #ifdef DEBUG_HPACK
 /* dump the whole dynamic header table */
@@ -111,7 +111,7 @@ void hpack_dht_dump(FILE *out, const struct hpack_dht *dht)
 
 	for (i = HPACK_SHT_SIZE; i < HPACK_SHT_SIZE + dht->used; i++) {
 		slot = (hpack_get_dte(dht, i - HPACK_SHT_SIZE + 1) - dht->dte);
-		fprintf(out, "idx=%d slot=%u name=<%s> value=<%s> addr=%u-%u\n",
+		fprintf(out, "idx=%u slot=%u name=<%s> value=<%s> addr=%u-%u\n",
 			i, slot,
 			istpad(name, hpack_idx_to_name(dht, i)).ptr,
 			istpad(value, hpack_idx_to_value(dht, i)).ptr,
