@@ -57,9 +57,8 @@ void quic_tls_keys_hexdump(struct buffer *buf,
 
 	if (!secs->aead || !secs->hp)
 		return;
-
-	aead_keylen = (size_t)EVP_CIPHER_key_length(secs->aead);
-	aead_ivlen = (size_t)EVP_CIPHER_iv_length(secs->aead);
+	aead_keylen = (size_t)QUIC_AEAD_key_length(secs->aead);
+	aead_ivlen = (size_t)QUIC_AEAD_iv_length(secs->aead);
 	hp_len = (size_t)EVP_CIPHER_key_length(secs->hp);
 
 	chunk_appendf(buf, "\n          key=");
@@ -200,7 +199,7 @@ static int quic_conn_enc_level_init(struct quic_conn *qc,
 	int ret = 0;
 	struct quic_enc_level *qel;
 
-	TRACE_ENTER(QUIC_EV_CONN_CLOSE, qc);
+	TRACE_ENTER(QUIC_EV_CONN_NEW, qc);
 
 	qel = pool_alloc(pool_head_quic_enc_level);
 	if (!qel)
@@ -244,11 +243,27 @@ static int quic_conn_enc_level_init(struct quic_conn *qc,
 			goto err;
 	}
 
-	LIST_APPEND(&qc->qel_list, &qel->list);
+	/* Ensure early-data encryption is not inserted at the end of this ->qel_list
+	 * list. This would perturbate the sender during handshakes. This latter adds
+	 * PADDING frames to datagrams from the last encryption level in this list,
+	 * for datagram with at least an ack-eliciting Initial packet inside.
+	 * But a QUIC server has nothing to send from this early-data encryption
+	 * level, contrary to the client.
+	 * Here early-data is added after the Initial encryption level which is
+	 * always already present.
+	 */
+	if (level == ssl_encryption_early_data) {
+		if (qc->iel)
+			LIST_APPEND(&qc->iel->list, &qel->list);
+		else
+			LIST_INSERT(&qc->qel_list, &qel->list);
+	}
+	else
+		LIST_APPEND(&qc->qel_list, &qel->list);
 	*el = qel;
 	ret = 1;
  leave:
-	TRACE_LEAVE(QUIC_EV_CONN_CLOSE, qc);
+	TRACE_LEAVE(QUIC_EV_CONN_NEW, qc);
 	return ret;
 
  err:
@@ -446,15 +461,15 @@ int quic_hkdf_expand_label(const EVP_MD *md,
  * ->hp_key is the key to be derived for header protection.
  * Obviouly these keys have the same size becaused derived with the same TLS cryptographic context.
  */
-int quic_tls_derive_keys(const EVP_CIPHER *aead, const EVP_CIPHER *hp,
+int quic_tls_derive_keys(const QUIC_AEAD *aead, const EVP_CIPHER *hp,
                          const EVP_MD *md, const struct quic_version *qv,
                          unsigned char *key, size_t keylen,
                          unsigned char *iv, size_t ivlen,
                          unsigned char *hp_key, size_t hp_keylen,
                          const unsigned char *secret, size_t secretlen)
 {
-	size_t aead_keylen = (size_t)EVP_CIPHER_key_length(aead);
-	size_t aead_ivlen = (size_t)EVP_CIPHER_iv_length(aead);
+	size_t aead_keylen = (size_t)QUIC_AEAD_key_length(aead);
+	size_t aead_ivlen = (size_t)QUIC_AEAD_iv_length(aead);
 	size_t hp_len = hp ? (size_t)EVP_CIPHER_key_length(hp) : 0;
 
 	if (aead_keylen > keylen || aead_ivlen > ivlen || hp_len > hp_keylen)
@@ -563,13 +578,18 @@ void quic_aead_iv_build(unsigned char *iv, size_t ivlen,
 /* Initialize the cipher context for RX part of <tls_ctx> QUIC TLS context.
  * Return 1 if succeeded, 0 if not.
  */
-int quic_tls_rx_ctx_init(EVP_CIPHER_CTX **rx_ctx,
-                         const EVP_CIPHER *aead, unsigned char *key)
+int quic_tls_rx_ctx_init(QUIC_AEAD_CTX **rx_ctx,
+                         const QUIC_AEAD *aead, unsigned char *key)
 {
-	EVP_CIPHER_CTX *ctx;
-	int aead_nid = EVP_CIPHER_nid(aead);
 
-	ctx = EVP_CIPHER_CTX_new();
+#ifdef QUIC_AEAD_API
+	QUIC_AEAD_CTX *ctx = EVP_AEAD_CTX_new(aead, key, EVP_AEAD_key_length(aead), EVP_AEAD_DEFAULT_TAG_LENGTH);
+	if (!ctx)
+		return 0;
+
+#else
+	int aead_nid = EVP_CIPHER_nid(aead);
+	QUIC_AEAD_CTX *ctx = EVP_CIPHER_CTX_new();
 	if (!ctx)
 		return 0;
 
@@ -580,12 +600,12 @@ int quic_tls_rx_ctx_init(EVP_CIPHER_CTX **rx_ctx,
 	    !EVP_DecryptInit_ex(ctx, NULL, NULL, key, NULL))
 		goto err;
 
+#endif
 	*rx_ctx = ctx;
-
 	return 1;
 
  err:
-	EVP_CIPHER_CTX_free(ctx);
+	QUIC_AEAD_CTX_free(ctx);
 	return 0;
 }
 
@@ -672,13 +692,18 @@ int quic_tls_aes_decrypt(unsigned char *out,
 /* Initialize the cipher context for TX part of <tls_ctx> QUIC TLS context.
  * Return 1 if succeeded, 0 if not.
  */
-int quic_tls_tx_ctx_init(EVP_CIPHER_CTX **tx_ctx,
-                         const EVP_CIPHER *aead, unsigned char *key)
+int quic_tls_tx_ctx_init(QUIC_AEAD_CTX **tx_ctx,
+                         const QUIC_AEAD *aead, unsigned char *key)
 {
-	EVP_CIPHER_CTX *ctx;
+#ifdef QUIC_AEAD_API
+	QUIC_AEAD_CTX *ctx = EVP_AEAD_CTX_new(aead, key, EVP_AEAD_key_length(aead), EVP_AEAD_DEFAULT_TAG_LENGTH);
+	if (!ctx)
+		return 0;
+
+#else
 	int aead_nid = EVP_CIPHER_nid(aead);
+	QUIC_AEAD_CTX *ctx = EVP_CIPHER_CTX_new();
 
-	ctx = EVP_CIPHER_CTX_new();
 	if (!ctx)
 		return 0;
 
@@ -688,13 +713,13 @@ int quic_tls_tx_ctx_init(EVP_CIPHER_CTX **tx_ctx,
 	     !EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_AEAD_SET_TAG, QUIC_TLS_TAG_LEN, NULL)) ||
 	    !EVP_EncryptInit_ex(ctx, NULL, NULL, key, NULL))
 		goto err;
+#endif
 
 	*tx_ctx = ctx;
-
 	return 1;
 
  err:
-	EVP_CIPHER_CTX_free(ctx);
+	QUIC_AEAD_CTX_free(ctx);
 	return 0;
 }
 
@@ -740,9 +765,18 @@ int quic_tls_tx_ctx_init(EVP_CIPHER_CTX **tx_ctx,
  */
 int quic_tls_encrypt(unsigned char *buf, size_t len,
                      const unsigned char *aad, size_t aad_len,
-                     EVP_CIPHER_CTX *ctx, const EVP_CIPHER *aead,
+                     QUIC_AEAD_CTX *ctx, const QUIC_AEAD *aead,
                      const unsigned char *iv)
 {
+#ifdef QUIC_AEAD_API
+	size_t outlen;
+
+	if (!EVP_AEAD_CTX_seal(ctx, buf, &outlen, len + EVP_AEAD_max_overhead(aead),
+	                       iv, QUIC_TLS_IV_LEN,
+	                       buf, len,
+	                       aad, aad_len))
+		return 0;
+#else
 	int outlen;
 	int aead_nid = EVP_CIPHER_nid(aead);
 
@@ -755,6 +789,9 @@ int quic_tls_encrypt(unsigned char *buf, size_t len,
 		!EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_AEAD_GET_TAG, QUIC_TLS_TAG_LEN, buf + len))
 		return 0;
 
+
+#endif
+
 	return 1;
 }
 
@@ -769,9 +806,20 @@ int quic_tls_encrypt(unsigned char *buf, size_t len,
  */
 int quic_tls_decrypt(unsigned char *buf, size_t len,
                      unsigned char *aad, size_t aad_len,
-                     EVP_CIPHER_CTX *ctx, const EVP_CIPHER *aead,
+                     QUIC_AEAD_CTX *ctx, const QUIC_AEAD *aead,
                      const unsigned char *key, const unsigned char *iv)
 {
+#ifdef QUIC_AEAD_API
+	size_t outlen;
+
+	if (!EVP_AEAD_CTX_open(ctx, buf, &outlen, len,
+	                       iv, QUIC_TLS_IV_LEN,
+	                       buf, len,
+	                       aad, aad_len))
+		return 0;
+
+#else
+
 	int outlen;
 	int aead_nid = EVP_CIPHER_nid(aead);
 
@@ -786,6 +834,8 @@ int quic_tls_decrypt(unsigned char *buf, size_t len,
 		 !EVP_DecryptFinal_ex(ctx, buf + outlen, &outlen)))
 		return 0;
 
+#endif
+
 	return 1;
 }
 
@@ -805,9 +855,20 @@ int quic_tls_decrypt(unsigned char *buf, size_t len,
 int quic_tls_decrypt2(unsigned char *out,
                       unsigned char *in, size_t len,
                       unsigned char *aad, size_t aad_len,
-                      EVP_CIPHER_CTX *ctx, const EVP_CIPHER *aead,
+                      QUIC_AEAD_CTX *ctx, const QUIC_AEAD *aead,
                       const unsigned char *key, const unsigned char *iv)
 {
+#ifdef QUIC_AEAD_API
+	size_t outlen;
+
+	if (!EVP_AEAD_CTX_open(ctx, out, &outlen, len,
+	                       iv, QUIC_TLS_IV_LEN,
+	                       in, len,
+	                       aad, aad_len))
+		return 0;
+
+#else
+
 	int outlen;
 	int aead_nid = EVP_CIPHER_nid(aead);
 
@@ -821,6 +882,7 @@ int quic_tls_decrypt2(unsigned char *out,
 	    (aead_nid != NID_aes_128_ccm &&
 	     !EVP_DecryptFinal_ex(ctx, out + outlen, &outlen)))
 		return 0;
+#endif
 
 	return 1;
 }
@@ -829,25 +891,54 @@ int quic_tls_decrypt2(unsigned char *out,
  * with <secret> which is not pseudo-random.
  * Return 1 if succeeded, 0 if not.
  */
+static inline int quic_do_tls_derive_token_secret(const EVP_MD *md, unsigned char *key, size_t keylen,
+                                                  unsigned char *iv, size_t ivlen,
+                                                  const unsigned char *salt, size_t saltlen,
+                                                  const unsigned char *secret, size_t secretlen,
+                                                  const unsigned char *klabel, size_t klabellen,
+                                                  const unsigned char *ivlabel, size_t ivlabellen)
+{
+	unsigned char tmpkey[QUIC_TLS_KEY_LEN];
+
+	if (!quic_hkdf_extract(md, tmpkey, sizeof tmpkey,
+	                       secret, secretlen, salt, saltlen) ||
+	    !quic_hkdf_expand(md, key, keylen, tmpkey, sizeof tmpkey,
+	                      klabel, klabellen) ||
+	    !quic_hkdf_expand(md, iv, ivlen, tmpkey, sizeof tmpkey,
+	                      ivlabel, ivlabellen))
+		return 0;
+
+	return 1;
+}
+
 int quic_tls_derive_retry_token_secret(const EVP_MD *md,
                                        unsigned char *key, size_t keylen,
                                        unsigned char *iv, size_t ivlen,
                                        const unsigned char *salt, size_t saltlen,
                                        const unsigned char *secret, size_t secretlen)
 {
-	unsigned char tmpkey[QUIC_TLS_KEY_LEN];
 	const unsigned char key_label[] = "retry token key";
 	const unsigned char iv_label[] = "retry token iv";
 
-	if (!quic_hkdf_extract(md, tmpkey, sizeof tmpkey,
-	                       secret, secretlen, salt, saltlen) ||
-	    !quic_hkdf_expand(md, key, keylen, tmpkey, sizeof tmpkey,
-	                      key_label, sizeof key_label - 1) ||
-	    !quic_hkdf_expand(md, iv, ivlen, tmpkey, sizeof tmpkey,
-	                      iv_label, sizeof iv_label - 1))
-		return 0;
+	return quic_do_tls_derive_token_secret(md, key, keylen, iv, ivlen,
+	                                       salt, saltlen, secret, secretlen,
+	                                       key_label, sizeof(key_label) - 1,
+	                                       iv_label, sizeof(iv_label) -1);
+}
 
-	return 1;
+int quic_tls_derive_token_secret(const EVP_MD *md,
+                                 unsigned char *key, size_t keylen,
+                                 unsigned char *iv, size_t ivlen,
+                                 const unsigned char *salt, size_t saltlen,
+                                 const unsigned char *secret, size_t secretlen)
+{
+	const unsigned char key_label[] = "token key";
+	const unsigned char iv_label[] = "token iv";
+
+	return quic_do_tls_derive_token_secret(md, key, keylen, iv, ivlen,
+	                                       salt, saltlen, secret, secretlen,
+	                                       key_label, sizeof(key_label) - 1,
+	                                       iv_label, sizeof(iv_label) -1);
 }
 
 /* Generate the AEAD tag for the Retry packet <pkt> of <pkt_len> bytes and
@@ -961,7 +1052,7 @@ int quic_tls_key_update(struct quic_conn *qc)
 
 	kp_trace.tx = nxt_tx;
 	if (nxt_rx->ctx) {
-		EVP_CIPHER_CTX_free(nxt_rx->ctx);
+		QUIC_AEAD_CTX_free(nxt_rx->ctx);
 		nxt_rx->ctx = NULL;
 	}
 
@@ -971,7 +1062,7 @@ int quic_tls_key_update(struct quic_conn *qc)
 	}
 
 	if (nxt_tx->ctx) {
-		EVP_CIPHER_CTX_free(nxt_tx->ctx);
+		QUIC_AEAD_CTX_free(nxt_tx->ctx);
 		nxt_tx->ctx = NULL;
 	}
 
@@ -995,7 +1086,7 @@ void quic_tls_rotate_keys(struct quic_conn *qc)
 {
 	struct quic_tls_ctx *tls_ctx = &qc->ael->tls_ctx;
 	unsigned char *curr_secret, *curr_iv, *curr_key;
-	EVP_CIPHER_CTX *curr_ctx;
+	QUIC_AEAD_CTX *curr_ctx;
 
 	TRACE_ENTER(QUIC_EV_CONN_RXPKT, qc);
 
